package org.litesoft.util;

import java.util.*;

// Copyright Status:
//
// All Software available from LiteSoft.org (including this file) is
// hereby released into the public domain.
//
// It is free!  As in, you may use it freely in both commercial and
// non-commercial applications, bundle it with your software
// distribution, include it on a CD-ROM, list the source code in a book,
// mirror the documentation at your own web site, or use it in any other
// way you see fit.
//
// NO Warranty!
//
// All software is provided "as is".
//
// There is ABSOLUTELY NO WARRANTY OF ANY KIND: not for the design, fitness
// (for a particular purpose), level of errors (or lack thereof), or
// applicability of this software.  The entire risk as to the quality
// and performance of this software is with you.  Should this software
// prove defective, you assume the cost of all necessary servicing, repair
// or correction.
//
// In no event unless required by applicable law or agreed to in writing
// will any party who created or may modify and/or redistribute this
// software, be liable to you for damages, including any general,
// special, incidental or consequential damages arising out of the use or
// inability to use this software (including but not limited to loss of
// data or data being rendered inaccurate or losses sustained by you or
// third parties or a failure of this software to operate with any
// other programs), even if such holder or other party has been advised
// of the possibility of such damages.
//
// NOTE: Should you discover a bug, have a recogmendation for a change, wish
// to submit modifications, or wish to add new classes/functionality,
// please email them to:
//
//        changes@litesoft.org
//

/**
 * An Array Helper class consisting completely of static methods.<p>
 *
 * Optimally, SUN/Java would have provided a seamless mechanism (or at
 * least an <i>import like mechanism</i>) to add behavior/functionality
 * to an existing class while preserving the syntactic cleanliness of the
 * usage.<p>
 *
 * An example from this file would be:<p>
 * <pre>
 *          ArrayHelper.toString( array , "XYZZY" ) ;
 *      could have been replaced with:
 *          array.toString( "XYZZY" ) ;
 * </pre><p>
 *
 * Instead, we are left to create this class (with its clumsy usage).<p>
 *
 * Each method's first parameter is the <i>Array</i>
 * that would have been automatically passed in as <i>this</i>.
 * Note: As these would all be <i>instance</i> methods, the first
 * parameter may NOT be <i>null</i>.<p>
 *
 * When the first parameter can be null, by convention, I place these
 * methods in a aaaUtil class.<p>
 *
 * See <a href="ArrayUtil.html">ArrayUtil</a><p>
 *
 * This class is non-instantiable.<p>
 *
 * @author  George Smith
 * @version 1.0 7/28/01
 *
 * @see ArrayUtil
 */

public class ArrayHelper
{
    private ArrayHelper()
    {
    }

    /**
     * Returns a debug/human friendly <i>pLabel</i>ed String that represents
     * the <i>pThis</i>.<p>
     *
     * @param   pThis   the Array to dump (!null).
     * @param   pLabel   the pLabel for the Array (null OK).<p>
     *
     * @return  A <i>pLabel</i>ed String that represents <i>pThis</i>.
     */
    public static String toString( Object[] pThis , String pLabel )
        throws NullPointerException
    {
        StringBuffer sb = new StringBuffer();
        if ( pLabel != null )
        {
            sb.append( pLabel );
            sb.append( ": " );
        }
        sb.append( pThis.length );
        sb.append( ' ' );
        String pThisClass = pThis.getClass().getComponentType().getName();
        // 01234567890
        if ( pThisClass.startsWith( "java.lang." ) )
            pThisClass = pThisClass.substring( 10 );
        sb.append( pThisClass );
        if ( pThis.length != 0 )
        {
            sb.append( '\n' );
            for ( int i = 0 ; i < pThis.length ; i++ )
            {
                Object entry = pThis[i];
                sb.append( "    [" );
                sb.append( (entry == null) ? "?null?" : entry.toString() );
                sb.append( "]\n" );
            }
        }
        return sb.toString();
    }

    /**
     * Returns a merging of all the elements of <i>pThis</i> into a <i>String</i>
     * with multiple elements seperated with <i>pSeperator</i>.<p>
     *
     * @param   pThis       the Array to merge (!null).
     * @param   pSeperator  the seperator to add between the Array elements
     *                      (!null).<p>
     *
     * @return  A merged <i>seperated</i> Array as a String.
     */
    public static String mergeWithSeperator( Object[] pThis , String pSeperator )
        throws NullPointerException
    {
        int siz , arrayLen = pThis.length;
        if ( arrayLen == 0 )
            return "";
        if ( arrayLen == 1 )
            return pThis[0].toString();

        String[] cache = new String[arrayLen];
        for ( int x = siz = 0 ; x < pThis.length ; x++ )
            siz += (cache[x] = pThis[x].toString()).length();

        StringBuffer sb = new StringBuffer( siz + ((arrayLen - 1) * pSeperator.length()) );

        sb.append( cache[0] );

        for ( int x = 1 ; x < arrayLen ; x++ )
        {
            sb.append( pSeperator );
            sb.append( cache[x] );
        }

        return sb.toString();
    }

    /**
     * Returns an array that is the result of <i>pToAppend</i> appended onto
     * <i>pThis</i>.<p>
     *
     * The array type of the returned array will be either an <tt>Object</tt>
     * array or an array of the type passed in as the <i>pThis</i> array type.
     * If the <i>pThis</i> array type is either <tt>Object[]</tt> or any element
     * of <i>pToAppend</i> can not be assigned to the <i>pThis</i> array type
     * (eg <tt>String[]</tt>), then the returned array type is <tt>Object[]</tt>.<p>
     *
     * @param   pThis       the Array to append on to (!null).
     * @param   pToAppend   the Array to append (null OK).<p>
     *
     * @return  An array that is the result of <i>pToAppend</i> appended onto
     *          <i>pThis</i>.  Or just <i>pThis</i> if <i>pToAppend</i> is null.
     */
    public static Object[] append( Object[] pThis , Object[] pToAppend )
        throws NullPointerException
    {
        Class arrayType = pThis.getClass().getComponentType(); // force null pointer Exception

        if ( pToAppend == null )
            return pThis;

        int otherLen = pToAppend.length;

        if ( otherLen == 0 )
            return pThis;

        int arrayLen = pThis.length;

        ArrayList retval = new ArrayList( arrayLen + otherLen );

        if ( arrayLen != 0 )
            retval.addAll( Arrays.asList( pThis ) );

        boolean makeObject = arrayType.getName().equals( Object.class.getName() );

        if ( makeObject )
            retval.addAll( Arrays.asList( pToAppend ) );
        else
            for ( int i = 0 ; i < otherLen ; i++ )
            {
                Object o = pToAppend[i];
                retval.add( o );
                if ( !arrayType.isAssignableFrom( o.getClass() ) )
                    makeObject = true;
            }

        return makeObject ? retval.toArray() : retval.toArray( pThis );
    }
}
